/*******************************************************************************
 * ============LICENSE_START========================================================================
 * ONAP : ccsdk feature sdnr wt
 * =================================================================================================
 * Copyright (C) 2019 highstreet technologies GmbH Intellectual Property. All rights reserved.
 * =================================================================================================
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 * ============LICENSE_END==========================================================================
 ******************************************************************************/

package org.onap.ccsdk.features.sdnr.wt.devicemanager.maintenance.impl;

import java.util.ArrayList;
import java.util.List;
import org.eclipse.jdt.annotation.NonNull;
import org.eclipse.jdt.annotation.Nullable;
import org.onap.ccsdk.features.sdnr.wt.common.HtAssert;
import org.onap.ccsdk.features.sdnr.wt.common.database.HtDatabaseClient;
import org.onap.ccsdk.features.sdnr.wt.database.EsDataObjectReaderWriter2;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.data.provider.rev190801.Entity;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.data.provider.rev190801.MaintenanceBuilder;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.data.provider.rev190801.MaintenanceEntity;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.data.provider.rev190801.NetworkElementConnectionBuilder;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.data.provider.rev190801.NetworkElementConnectionEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HtDatabaseMaintenanceService {

    private static final Logger LOG = LoggerFactory.getLogger(HtDatabaseMaintenanceService.class);

    private final EsDataObjectReaderWriter2<MaintenanceEntity> maintenanceRW;
    private final EsDataObjectReaderWriter2<NetworkElementConnectionEntity> requiredNeRW;

    HtDatabaseMaintenanceService(@NonNull HtDatabaseClient client) throws ClassNotFoundException {
        HtAssert.nonnull(client);

        // Create control structure
        maintenanceRW = new EsDataObjectReaderWriter2<>(client, Entity.Maintenancemode, MaintenanceEntity.class,
                MaintenanceBuilder.class).setEsIdAttributeName("_id");

        requiredNeRW = new EsDataObjectReaderWriter2<>(client, Entity.NetworkelementConnection,
                NetworkElementConnectionEntity.class, NetworkElementConnectionBuilder.class)
                        .setEsIdAttributeName("_id");

    }

    /**
     * Get existing object for mountpoint to manage maintenance mode
     * @return Object with configuration
     */
    @Nullable MaintenanceEntity getMaintenance(String mountpointId) {
        MaintenanceEntity deviceMaintenanceMode = null;
        if (maintenanceRW != null || mountpointId != null) {
            deviceMaintenanceMode = maintenanceRW.read(mountpointId);
        }
        return deviceMaintenanceMode;
    }

    MaintenanceEntity setMaintenance(MaintenanceEntity m) {
        if (maintenanceRW != null) {
           if (maintenanceRW.write(m, m.getNodeId() ) == null) {
                throw new IllegalArgumentException("Problem writing to database: "+m.getId());
            }
           LOG.info("Wrote maintenance object {}", m.toString());
        }
        return m;
    }

    List<MaintenanceEntity> getAll() {
        return maintenanceRW != null ? maintenanceRW.doReadAll().getHits() : new ArrayList<>();
    }

    MaintenanceEntity createIfNotExists(String mountpointId) {
        MaintenanceEntity deviceMaintenanceMode = null;
        if (maintenanceRW != null) {
            deviceMaintenanceMode = maintenanceRW.read(mountpointId);
            if (deviceMaintenanceMode == null) {
                LOG.debug("creating empty maintenance object in database");
                deviceMaintenanceMode = MaintenanceCalculator.getDefaultMaintenance(mountpointId);
                maintenanceRW.write(deviceMaintenanceMode, mountpointId);
            } else {
                LOG.debug("maintenance object already exists in database");
            }
        } else {
            LOG.warn("cannot create maintenance obj. db reader/writer is null");
        }
        return deviceMaintenanceMode;
    }

    void deleteIfNotRequired(String mountPointNodeName) {

        if (!this.isRequireNe(mountPointNodeName)) {
            if (maintenanceRW != null) {
                LOG.debug("removing maintenance object in database for " + mountPointNodeName);
                maintenanceRW.remove(mountPointNodeName);
            } else {
                LOG.warn("cannot create maintenance obj. db reader/writer is null");
            }
        }

    }


    // -- Private
    /**
     * Check in required ne if entry exists for mountpointNodeName
     *
     * @param mountPointNodeName
     * @return
     */
    private boolean isRequireNe(String mountPointNodeName) {
        NetworkElementConnectionEntity ne = null;
        if (requiredNeRW != null) {
            LOG.debug("searching for entry in required-networkelement for " + mountPointNodeName);
            ne = requiredNeRW.read(mountPointNodeName);
        } else {
            LOG.warn("cannot read db. no db reader writer initialized");
        }
        return ne != null;
    }

}
